



clang --version
IF %ERRORLEVEL% NEQ 0 exit /B 1
clang-cl --version
IF %ERRORLEVEL% NEQ 0 exit /B 1
clang-cpp --version
IF %ERRORLEVEL% NEQ 0 exit /B 1
if not exist %LIBRARY_LIB%\\clang\\10.0.0\\include\\stdatomic.h exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
